﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using ELTE.TodoList.Persistence;
using ELTE.TodoList.DTO;
using ELTE.TodoList.Persistence.Services;
using ELTE.TodoList.WebApi.Controllers;
using Xunit;
using AutoMapper;
using ELTE.TodoList.WebApi.MappingConfigurations;

namespace ELTE.TodoList.WebApi.Tests
{
    public class ListsControllerTest : IDisposable
    {
        private readonly TodoListDbContext _context;
        private readonly TodoListService _service;
        private readonly ListsController _controller;
        private readonly IMapper _mapper;

        public ListsControllerTest()
        {
            var options = new DbContextOptionsBuilder<TodoListDbContext>()
                .UseInMemoryDatabase("TestDb")
                .Options;

            _context = new TodoListDbContext(options);
            TestDbInitializer.Initialize(_context);

            /* 
             * Ezzel az utasítással elengedjük az adatbázis objektumainak követését (tracking).
             * Ez a listák átnevezésének teszteléséhez szükséges, mivel egyébként a
             * PutList megpróbálna új objektumot létrehozni az adatbázisban.
             */
            _context.ChangeTracker.Clear();
            _service = new TodoListService(_context);

            var config = new MapperConfiguration(cfg =>
            {
                cfg.AddProfile(new ListProfile());
                cfg.AddProfile(new ItemProfile());
                cfg.AddProfile(new ItemDtoProfile());
                cfg.AddProfile(new ListDtoProfile());
            });
            _mapper = new Mapper(config);
            _controller = new ListsController(_service, _mapper);
        }

        public void Dispose()
        {
            _context.Database.EnsureDeleted();
            _context.Dispose();
        }

        [Fact]
        public void GetListsTest()
        {
            // Act
            var result = _controller.GetLists();

            // Assert
            var content = Assert.IsAssignableFrom<IEnumerable<ListDto>>(result.Value);
            Assert.Equal(3, content.Count());
        }

        [Theory]
        [InlineData(1)]
        [InlineData(2)]
        [InlineData(3)]
        public void GetListByIdTest(Int32 id)
        {
            // Act
            var result = _controller.GetList(id);

            // Assert
            var content = Assert.IsAssignableFrom<ListDto>(result.Value);
            Assert.Equal(id, content.Id);
        }

        [Fact]
        public void GetInvalidListTest()
        {
            // Arrange
            var id = 4;

            // Act
            var result = _controller.GetList(id);

            // Assert
            Assert.IsAssignableFrom<NotFoundResult>(result.Result);
        }

        [Theory]
        [InlineData(1)]
        [InlineData(2)]
        [InlineData(3)]
        public void PutListTest(Int32 id)
        {
            // Arrange
            string newName = "Updated list";
            var listDto = new ListDto { Id = id, Name =  newName};

            // Act
            var result = _controller.PutList(id, listDto);

            // Assert
            var requestResult = Assert.IsAssignableFrom<OkResult>(result);
            var updatedList = _controller.GetList(id);
            Assert.Equal(updatedList?.Value?.Name, newName);
        }

        [Fact]
        public void PostListTest()
        {
            // Arrange
            var newList = new ListDto { Name = "New test list" };
            var count = _context.Lists.Count();

            // Act
            var result = _controller.PostList(newList);

            // Assert
            var objectResult = Assert.IsAssignableFrom<CreatedAtActionResult>(result.Result);
            var content = Assert.IsAssignableFrom<ListDto>(objectResult.Value);
            Assert.Equal(count + 1, _context.Lists.Count());
        }
    }
}
